////////////////////////////////////////////////////////////////////////////////
//                            MESH_SceneGraphTypes.inl                        //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace MESH {

template<class VData, class EData, class FData, bool readOnly>
EntityMFVertex<VData, EData, FData, readOnly>::EntityMFVertex() 
: std::vector<SFVertex<VData, EData, FData, readOnly> *>()
{
}

template<class VData, class EData, class FData>
EntityMFVertex<VData, EData, FData, false>::EntityMFVertex() 
: std::list<SFVertex<VData, EData, FData, false> *>()
{
}



template<class VData, class EData, class FData, bool readOnly>
MeshMFVertex<VData, EData, FData, readOnly>::MeshMFVertex() 
: std::vector<SFVertex<VData, EData, FData, readOnly> *>()
{
}

template<class VData, class EData, class FData, bool readOnly>
std::pair<typename std::vector<SFVertex<VData, EData, FData, readOnly> *>::iterator, bool> MeshMFVertex<VData, EData, FData, readOnly>::insert(const std::pair<unsigned int, SFVertex<VData, EData, FData, readOnly> *> &x)
{
  if (x.first >= size())
    resize(x.first + 1, 0);
  
  typename std::vector<SFVertex<VData, EData, FData, readOnly> *>::iterator it = begin();
  it += x.first;
  *it = x.second;
    
  return std::pair<typename std::vector<SFVertex<VData, EData, FData, readOnly> *>::iterator, bool>(it, true);
}

template<class VData, class EData, class FData, bool readOnly>
typename std::vector<SFVertex<VData, EData, FData, readOnly> *>::iterator MeshMFVertex<VData, EData, FData, readOnly>::find(SFVertex<VData, EData, FData, readOnly> *const &element) const
{
  return std::find(begin(), end(), element);
}

template<class VData, class EData, class FData>
MeshMFVertex<VData, EData, FData, false>::MeshMFVertex() 
: std::map<unsigned int, SFVertex<VData, EData, FData, false> *>()
{
}

template<class VData, class EData, class FData>
typename std::map<unsigned int, SFVertex<VData, EData, FData, false> *>::iterator MeshMFVertex<VData, EData, FData, false>::find(SFVertex<VData, EData, FData, false> *const &element)
{
  typename std::map<unsigned int, SFVertex<VData, EData, FData, false> *>::iterator it = std::map<unsigned int, SFVertex<VData, EData, FData, false> *>::find(element->getIndex());
  if ((*it).second == element)
    return it;
  return end();  
}

template<class VData, class EData, class FData>
typename std::map<unsigned int, SFVertex<VData, EData, FData, false> *>::const_iterator MeshMFVertex<VData, EData, FData, false>::find(SFVertex<VData, EData, FData, false> *const &element) const
{
  typename std::map<unsigned int, SFVertex<VData, EData, FData, false> *>::const_iterator it = std::map<unsigned int, SFVertex<VData, EData, FData, false> *>::find(element->getIndex());
  if ((*it).second == element)
    return it;
  return end();  
}



template<class EData, class FData, class VData, bool readOnly>
EntityMFEdge<EData, FData, VData, readOnly>::EntityMFEdge() 
: std::vector<SFEdge<EData, FData, VData, readOnly> *>() 
{
}

template<class EData, class FData, class VData>
EntityMFEdge<EData, FData, VData, false>::EntityMFEdge() 
: std::list<SFEdge<EData, FData, VData, false> *>()
{
}

template<class EData, class FData, class VData, bool readOnly>
EntityMFEdge<EData, FData, VData, readOnly>::EntityMFEdge(const std::vector<SFEdge<EData, FData, VData, readOnly> *> &vector, bool ccw)
: std::vector<SFEdge<EData, FData, VData, readOnly> *>(vector)
{
  if (!ccw)
  {
    unsigned int last = vector.size() - 1;
    unsigned int mid = (last + 1)/2;
    for (unsigned int i = 0; i < mid; ++i)
    {
      SFEdge<EData, FData, VData, readOnly> *temp = (*this)[i];
      (*this)[i] = (*this)[last - i];
      (*this)[last - i] = temp;
    }
  }  
}



template<class EData, class FData, class VData, bool readOnly>
MeshMFEdge<EData, FData, VData, readOnly>::MeshMFEdge() 
: std::vector<SFEdge<EData, FData, VData, readOnly> *>() 
{
}

template<class EData, class FData, class VData, bool readOnly>
std::pair<typename std::vector<SFEdge<EData, FData, VData, readOnly> *>::iterator, bool> MeshMFEdge<EData, FData, VData, readOnly>::insert(const typename std::vector<SFEdge<EData, FData, VData, readOnly> *>::value_type &x)
{
  push_back(x);
  return std::pair<typename std::vector<SFEdge<EData, FData, VData, readOnly> *>::iterator, bool>(end(), true);
}

template<class EData, class FData, class VData, bool readOnly>
typename std::vector<SFEdge<EData, FData, VData, readOnly> *>::iterator MeshMFEdge<EData, FData, VData, readOnly>::find(SFEdge<EData, FData, VData, readOnly> *const &element) const
{
  return std::find(begin(), end(), element);
}

template<class EData, class FData, class VData>
MeshMFEdge<EData, FData, VData, false>::MeshMFEdge() 
: std::set<SFEdge<EData, FData, VData, false> *>()
{
}

template<class EData, class FData, class VData>
SFEdge<EData, FData, VData, false> *const &MeshMFEdge<EData, FData, VData, false>::operator[](const unsigned int &n) const
{
  typename std::set<SFEdge<EData, FData, VData, false> *>::const_iterator it = begin();
  for (unsigned int i = 0; i < n; ++i)
    ++it;
  
  return *it;  
}


template<class FData, class VData, class EData, bool readOnly>
EntityMFFace<FData, VData, EData, readOnly>::EntityMFFace() 
: std::vector<SFFace<FData, VData, EData, readOnly> *>() 
{
}

template<class FData, class VData, class EData>
EntityMFFace<FData, VData, EData, false>::EntityMFFace() 
: std::list<SFFace<FData, VData, EData, false> *>() 
{
}



template<class FData, class VData, class EData, bool readOnly>
MeshMFFace<FData, VData, EData, readOnly>::MeshMFFace() 
: std::vector<SFFace<FData, VData, EData, readOnly> *>() 
{
}

template<class FData, class VData, class EData, bool readOnly>
std::pair<typename std::vector<SFFace<FData, VData, EData, readOnly> *>::iterator, bool> MeshMFFace<FData, VData, EData, readOnly>::insert(const std::pair<unsigned int, SFFace<FData, VData, EData, readOnly> *> &x)
{
  if (x.first >= size())
    resize(x.first + 1, 0);
  
  typename std::vector<SFFace<FData, VData, EData, readOnly> *>::iterator it = begin();
  it += x.first;
  *it = x.second;
    
  return std::pair<typename std::vector<SFFace<FData, VData, EData, readOnly> *>::iterator, bool>(it, true);
}

template<class FData, class VData, class EData, bool readOnly>
typename std::vector<SFFace<FData, VData, EData, readOnly> *>::iterator MeshMFFace<FData, VData, EData, readOnly>::find(SFFace<FData, VData, EData, readOnly> *const &element)
{
  return std::find(begin(), end(), element);
}

template<class FData, class VData, class EData, bool readOnly>
typename std::vector<SFFace<FData, VData, EData, readOnly> *>::const_iterator MeshMFFace<FData, VData, EData, readOnly>::find(SFFace<FData, VData, EData, readOnly> *const &element) const
{
  return std::find(begin(), end(), element);
}

template<class FData, class VData, class EData>
MeshMFFace<FData, VData, EData, false>::MeshMFFace() 
: std::map<unsigned int, SFFace<FData, VData, EData, false> *>() 
{
}

template<class FData, class VData, class EData>
typename std::map<unsigned int, SFFace<FData, VData, EData, false> *>::iterator MeshMFFace<FData, VData, EData, false>::find(SFFace<FData, VData, EData, false> *const &element)
{
  typename std::map<unsigned int, SFFace<FData, VData, EData, false> *>::iterator it = std::map<unsigned int, SFFace<FData, VData, EData, false> *>::find(element->getIndex());
  if ((*it).second == element)  
    return it;
  return end();  
}

template<class FData, class VData, class EData>
typename std::map<unsigned int, SFFace<FData, VData, EData, false> *>::const_iterator MeshMFFace<FData, VData, EData, false>::find(SFFace<FData, VData, EData, false> *const &element) const
{
  typename std::map<unsigned int, SFFace<FData, VData, EData, false> *>::const_iterator it = std::map<unsigned int, SFFace<FData, VData, EData, false> *>::find(element->getIndex());
  if ((*it).second == element)  
    return it;
  return end(); 
}

}
}

