#include "X3D_PointSet.h"
#include "X3D_X3DColorNode.h"
#include "X3D_X3DCoordinateNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

PointSet::PointSet()
: X3DGeometryNode(), _color(0), _coord(0)
{
  defineTypeName("PointSet", "Rendering");
}

PointSet::PointSet(X3DColorNode *color, X3DCoordinateNode *coord)
: X3DGeometryNode(), _color(0), _coord(0)
{
  defineTypeName("PointSet", "Rendering");
  
  setColor(color);
  setCoord(coord);
}

PointSet::PointSet(const PointSet &P)
: X3DGeometryNode(P), _color(0), _coord(0)
{
  setColor(static_cast<X3DColorNode *>(P._color));
  setCoord(static_cast<X3DCoordinateNode *>(P._coord));
}

SFAbstractNode PointSet::clone() const
{
  return new PointSet(*this);
}

PointSet::~PointSet()
{
  removeParentFromChild(this, _color);
  removeParentFromChild(this, _coord);
}

//modifiers
void PointSet::setColor(X3DColorNode *color)
{
  removeParentFromChild(this, _color);
  _color = color;
  addParentToChild(this, _color);
}

void PointSet::setCoord(X3DCoordinateNode *coord)
{
  removeParentFromChild(this, _coord);
  _coord = coord;
  addParentToChild(this, _coord);
}

MFAbstractNode PointSet::getChildrenList() const
{
  MFAbstractNode childrenList;
  if (_color != 0)
    childrenList.push_back(_color);
  if (_coord != 0)    
  childrenList.push_back(_coord);
    
  return childrenList; 
}

bool PointSet::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    if (_color != 0)
      return false;
      
    setColor(static_cast<X3DColorNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    if (_coord != 0)
      return false;
      
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  if (N != 0)
    cx3d << "X3D::PointSet::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::PointSet::addChild : cannot add a NULL node!" << endl;
     
  
  return false;
}

bool PointSet::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    setColor(static_cast<X3DColorNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  if (N != 0)
    cx3d << "X3D::PointSet::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::PointSet::setChild : cannot add a NULL node!" << endl;
     
  
  return false;
}

bool PointSet::removeChild(const SFAbstractNode &N)
{
  if (_color == N)
  {
    setColor(0);
    return true;
  } 
   
  if (_coord == N)
  {
    setCoord(0);
    return true;
  } 
  
  return false;
}

}
}
