#include "X3D_TriangleStripSet.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

TriangleStripSet::TriangleStripSet()
: X3DComposedGeometryNode(), _convex(true), _creaseAngle(0.0f)
{
  defineTypeName("TriangleStripSet", "Rendering");
}

TriangleStripSet::TriangleStripSet(X3DColorNode *color, X3DCoordinateNode *coord, X3DNormalNode *normal, X3DTextureCoordinateNode *texCoord, const SFBool &ccw, const SFBool &colorPerVertex, const SFBool &convex, const SFFloat &creaseAngle, const SFBool &normalPerVertex, const SFBool &solid, const MFInt32 &stripCount)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
_convex(convex), _creaseAngle(creaseAngle), _stripCount(stripCount)
{
  defineTypeName("TriangleStripSet", "Rendering");
}

TriangleStripSet::TriangleStripSet(const TriangleStripSet &T)
: X3DComposedGeometryNode(T), _convex(T._convex), _creaseAngle(T._creaseAngle), _stripCount(T._stripCount)
{
}

SFAbstractNode TriangleStripSet::clone() const
{
  return new TriangleStripSet(*this);
}

TriangleStripSet::~TriangleStripSet()
{
}

void TriangleStripSet::setConvex(const SFBool &convex)
{
  _convex = convex;
}

void TriangleStripSet::setCreaseAngle(const SFFloat &creaseAngle)
{
  _creaseAngle = creaseAngle;
}

void TriangleStripSet::setStripCount(const MFInt32 &stripCount)
{
  _stripCount  = stripCount;
}

void TriangleStripSet::load(const X3DFileElement *element)
{
  X3DComposedGeometryNode::load(element);
  
  int index;
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    _convex = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    _creaseAngle = element->getAttribute(index).toFloat(); 
    
  index = element->getIndexAttribute("stripCount");  
  if (index != -1)
    _stripCount = MFInt32(element->getAttribute(index));  
}

SFString &TriangleStripSet::write(SFString &output) const
{
  X3DComposedGeometryNode::write(output);
  
  if (!_convex)  
    output += " convex=\"FALSE\"";
  if (_creaseAngle != 0.0f)
    output += " creaseAngle=\"" + toSFString(_creaseAngle) + "\"";
  if (!_stripCount.empty())
    output += " stripCount=\"" + toSFString(_stripCount) + "\"";
    
  return output;
}

}
}
