#include "X3D_X3DComposedGeometryNode.h"
#include "X3D_X3DColorNode.h"
#include "X3D_X3DCoordinateNode.h"
#include "X3D_X3DNormalNode.h"
#include "X3D_X3DTextureCoordinateNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

X3DComposedGeometryNode::X3DComposedGeometryNode()
: X3DGeometry3DNode(), _color(0), _coord(0), _normal(0), _texCoord(0), _ccw(true), _colorPerVertex(true), _normalPerVertex(true), _solid(true)
{
  defineTypeName("X3DComposedGeometryNode");
}

X3DComposedGeometryNode::X3DComposedGeometryNode(X3DColorNode *color, X3DCoordinateNode *coord, X3DNormalNode *normal, X3DTextureCoordinateNode *texCoord, const SFBool &ccw, const SFBool &colorPerVertex, const SFBool &normalPerVertex, const SFBool &solid)
: X3DGeometry3DNode(), _color(0), _coord(0), _normal(0), _texCoord(0), _ccw(ccw), _colorPerVertex(colorPerVertex), _normalPerVertex(normalPerVertex), _solid(solid)
{
  defineTypeName("X3DComposedGeometryNode");
  
  setColor(static_cast<X3DColorNode *>(color));
  setCoord(static_cast<X3DCoordinateNode *>(coord));
  setNormal(static_cast<X3DNormalNode *>(normal));
  setTexCoord(static_cast<X3DTextureCoordinateNode *>(texCoord));  
}
           
X3DComposedGeometryNode::X3DComposedGeometryNode(const X3DComposedGeometryNode &N)
: X3DGeometry3DNode(N), _color(0), _coord(0), _normal(0), _texCoord(0), _ccw(N._ccw), _colorPerVertex(N._colorPerVertex), _normalPerVertex(N._normalPerVertex), _solid(N._solid)
{
  setColor(static_cast<X3DColorNode *>(N._color));
  setCoord(static_cast<X3DCoordinateNode *>(N._coord));
  setNormal(static_cast<X3DNormalNode *>(N._normal));
  setTexCoord(static_cast<X3DTextureCoordinateNode *>(N._texCoord));  
}

X3DComposedGeometryNode::~X3DComposedGeometryNode()
{
  removeParentFromChild(this, _color);
  removeParentFromChild(this, _coord);
  removeParentFromChild(this, _normal);
  removeParentFromChild(this, _texCoord);
}

void X3DComposedGeometryNode::setColor(X3DColorNode *color)
{
  removeParentFromChild(this, _color);
  _color = color;
  addParentToChild(this, _color);
}

void X3DComposedGeometryNode::setCoord(X3DCoordinateNode *coord)
{
  removeParentFromChild(this, _coord);
  _coord = coord;
  addParentToChild(this, _coord);
}

void X3DComposedGeometryNode::setNormal(X3DNormalNode *normal)
{
  removeParentFromChild(this, _normal);
  _normal = normal;
  addParentToChild(this, _normal);
}

void X3DComposedGeometryNode::setTexCoord(X3DTextureCoordinateNode *texCoord)
{
  removeParentFromChild(this, _texCoord);
  _texCoord = texCoord;
  addParentToChild(this, _texCoord);
}

void X3DComposedGeometryNode::setCcw(const SFBool &ccw)
{
  _ccw = ccw;
}

void X3DComposedGeometryNode::setColorPerVertex(const SFBool &colorPerVertex)
{
  _colorPerVertex = colorPerVertex;
}

void X3DComposedGeometryNode::setNormalPerVertex(const SFBool &normalPerVertex)
{
  _normalPerVertex = normalPerVertex;
}

void X3DComposedGeometryNode::setSolid(const SFBool &solid)
{
  _solid = solid;
}

MFAbstractNode X3DComposedGeometryNode::getChildrenList() const
{
  MFAbstractNode childrenList;
  if (_color != 0)
    childrenList.push_back(_color);
  if (_coord != 0)    
    childrenList.push_back(_coord);
  if (_coord != 0)    
    childrenList.push_back(_normal);
  if (_coord != 0)    
    childrenList.push_back(_texCoord);  
    
  return childrenList; 
}

bool X3DComposedGeometryNode::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    if (_color != 0)
      return false;
    
    setColor(static_cast<X3DColorNode *>(N)); 
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    if (_coord != 0)
      return false;
      
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DNormalNode *>(N) != 0)
  {
    if (_normal != 0)
      return false;
      
    setNormal(static_cast<X3DNormalNode *>(N));   
    return true;
  }
  
  if (dynamic_cast<X3DTextureCoordinateNode *>(N) != 0)
  {
    if (_texCoord != 0)
      return false;
    
    setTexCoord(static_cast<X3DTextureCoordinateNode *>(N));   
    return true;    
  }
  if (N != 0)
    cx3d << "X3D::X3DComposedGeometryNode::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::X3DComposedGeometryNode::addChild : cannot add a NULL node!" << endl;
   
  
  return false;
}

bool X3DComposedGeometryNode::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    setColor(static_cast<X3DColorNode *>(N)); 
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  if (dynamic_cast<X3DNormalNode *>(N) != 0)
  {
    setNormal(static_cast<X3DNormalNode *>(N));  
    return true;
  }
  
  if (dynamic_cast<X3DTextureCoordinateNode *>(N) != 0)
  {
    setTexCoord(static_cast<X3DTextureCoordinateNode *>(N));
    return true;    
  }
  if (N != 0)
    cx3d << "X3D::X3DComposedGeometryNode::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  else
    cx3d << "X3D::X3DComposedGeometryNode::setChild : cannot add a NULL node!" << endl;
  
  return false;
}

bool X3DComposedGeometryNode::removeChild(const SFAbstractNode &N)
{
  if (_color == N)
  {
    setColor(0);
    return true;
  } 
   
  if (_coord == N)
  {
    setCoord(0);
    return true;
  } 

  if (_normal == N)
  {
    setNormal(0);
    return true;
  } 
   
  if (_texCoord == N)
  {
    setTexCoord(0);
    return true;
  } 
  
  cx3d << "X3DComposedGeometryNode::removeChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;
}

void X3DComposedGeometryNode::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("ccw");
  if (index != -1)
    _ccw = (element->getAttribute(index).upper() == "TRUE");
  
  index = element->getIndexAttribute("colorPerVertex");  
  if (index != -1)
    _colorPerVertex = (element->getAttribute(index).upper() == "TRUE");  

  index = element->getIndexAttribute("normalPerVertex");
  if (index != -1)
    _normalPerVertex = (element->getAttribute(index).upper() == "TRUE");  
  
  index = element->getIndexAttribute("solid");  
  if (index != -1)
    _solid = (element->getAttribute(index).upper() == "TRUE");
}

SFString &X3DComposedGeometryNode::write(SFString &output) const
{
  if (!_ccw)
    output += " ccw=\"FALSE\"";
  if (!_colorPerVertex)  
    output += " colorPerVertex=\"FALSE\"";
  if (!_normalPerVertex)
    output += " normalPerVertex=\"FALSE\"";
  if (!_solid)  
    output += " solid=\"FALSE\"";  
  
  return output;
}

}
}
