#include "X3D_SceneSaverWriteCoreVisitor.h"
#include "X3D_Scene.h"
#include "X3D_X3DNode.h"
#include "X3DTypesToSFString.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaverWriteCoreVisitor::SceneSaverWriteCoreVisitor()
: CoreVisitor()
{
  // Enter functions.
  defineEnterFunction(&SceneSaverWriteCoreVisitor::enterX3DNode);

  // Walk on functions.
  defineWalkOnFunction(&SceneSaverWriteCoreVisitor::walkOnX3DNode);

  // Leave functions.
  defineLeaveFunction(&SceneSaverWriteCoreVisitor::leaveX3DNode);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<SceneSaverStateVariables>();
}

SceneSaverWriteCoreVisitor::~SceneSaverWriteCoreVisitor()
{
}

void SceneSaverWriteCoreVisitor::enterX3DNode(X3DNode *N) const
{
  stateVariables->newLine();
  stateVariables->pushCurrentLine();
  SFString &currentLine = stateVariables->getCurrentLine();
  
  if (stateVariables->findNode(N))
  {
    stateVariables->pushHasUSE(true);
    currentLine += "<" + N->getTypeName() + " USE=\"" + N->getName() + "\"";
  }
  else
  {
    stateVariables->pushHasUSE(false);    
    if (N->getName() != "")
      currentLine += "<" + N->getTypeName() + " DEF=\"" + N->getName() + "\"";
    else
      currentLine += "<" + N->getTypeName();  
    
    N->write(currentLine);  
    stateVariables->addNode(N);
  }
      
  stateVariables->addTab();
}

bool SceneSaverWriteCoreVisitor::walkOnX3DNode(X3DNode *, SFAbstractNode) const
{
  //return true;
  return !(stateVariables->getHasUSE());  
}

void SceneSaverWriteCoreVisitor::leaveX3DNode(X3DNode *N) const
{
  stateVariables->removeTab();
  
  if (stateVariables->hasChild())
    stateVariables->getOpeningLine() += "/>";  
  else
  {
    stateVariables->getOpeningLine() += ">";
    stateVariables->newLine();
    stateVariables->getCurrentLine() += "</" + N->getTypeName() + ">";
  }  
  
  stateVariables->popLine();
  stateVariables->popHasUSE();  
}

}
}
