#include "X3DTypesToSFString.h"

using namespace X3DTK;

SFString X3DTK::toSFString(const SFString &DEF, const SFString &USE)
{
  SFString res("");
  if (DEF != SFString(""))
    res += SFString(" DEF=\"") + DEF + SFString("\"");
  if (USE != SFString(""))
    res += SFString(" USE=\"") + USE + SFString("\""); 
  return res;   
}

SFString X3DTK::toSFString(const MFString &ms)
{
  SFString res("");
  if (!ms.empty())
  {
    for (unsigned int i = 0; i < ms.size() - 1; ++i)
      res += "\"" + ms[i] + SFString("\", ");
  
    res += "\"" + ms.back() + "\"";        
  }  
  return res;
}

SFString X3DTK::toSFString(const SFBool b)
{
  if (b)
    return "TRUE";
  return "FALSE";  
}

SFString X3DTK::toSFString(const SFInt32 i)
{
  return SFString::number(i);
}

SFString X3DTK::toSFString(const MFInt32 &mi)
{
  SFString res("");
  char buffer[20];
  
  if (!mi.empty())
  {
    for (unsigned int i = 0; i < mi.size() - 1; ++i)
    {
      sprintf(buffer, "%d", mi[i]);
      res += buffer;
      res += " ";
    }
    sprintf(buffer, "%d", mi.back());
    res += buffer;   
  }  
  return res;
}

SFString X3DTK::toSFString(const SFFloat f)
{
  return SFString::number(f);
}

SFString X3DTK::toSFString(const MFFloat &mf)
{
  SFString res("");
  char buffer[20];
  
  if (!mf.empty())
  {
    for (unsigned int i = 0; i < mf.size() - 1; ++i)
    {
      sprintf(buffer, "%.6g", mf[i]);
      res += buffer;
      res += " ";
    }
    sprintf(buffer, "%.6g", mf.back());
    res += buffer;   
  }  
  return res;
}

SFString X3DTK::toSFString(const SFDouble d)
{
  return SFString::number(d);
} 

SFString X3DTK::toSFString(const MFDouble &md)
{
  SFString res("");
  char buffer[50];
  
  if (!md.empty())
  {
    for (unsigned int i = 0; i < md.size() - 1; ++i)
    {
      sprintf(buffer, "%.15g", md[i]);
      res += buffer;
      res += " ";
    }
    sprintf(buffer, "%.15g", md.back());
    res += buffer;   
  }  
  return res;
}

SFString X3DTK::toSFString(const SFColorRGBA c)
{
  return SFString::number(c.r) + SFString(" ") 
       + SFString::number(c.g) + SFString(" ")
       + SFString::number(c.b) + SFString(" ")
       + SFString::number(c.a);
}

SFString X3DTK::toSFString(const MFColorRGBA &mc)
{
  SFString res("");
  char buffer[20];
  
  if (!mc.empty())
  {
    for (unsigned int i = 0; i < mc.size() - 1; ++i)
    {
      sprintf(buffer, "%.6g", mc[i].r);
      res += buffer;
      res += " ";
      sprintf(buffer, "%.6g", mc[i].g);
      res += buffer;
      res += " ";
      sprintf(buffer, "%.6g", mc[i].b);
      res += buffer;
      res += " ";
      sprintf(buffer, "%.6g", mc[i].a);
      res += buffer;
      res += ", ";
    }
    sprintf(buffer, "%.6g", mc.back().r);
    res += buffer;
    res += " ";
    sprintf(buffer, "%.6g", mc.back().g);
    res += buffer;
    res += " ";
    sprintf(buffer, "%.6g", mc.back().b);
    res += buffer;
    res += " ";
    sprintf(buffer, "%.6g", mc.back().a);
    res += buffer;
  }  
  return res;
}

SFString X3DTK::toSFString(const SFColor c)
{
  return SFString::number(c.r) + SFString(" ") 
       + SFString::number(c.g) + SFString(" ")
       + SFString::number(c.b);
}

SFString X3DTK::toSFString(const MFColor &mc)
{
  SFString res("");
  char buffer[20];
  
  if (!mc.empty())
  {
    for (unsigned int i = 0; i < mc.size() - 1; ++i)
    {
      sprintf(buffer, "%.6g", mc[i].r);
      res += buffer;
      res += " ";
      sprintf(buffer, "%.6g", mc[i].g);
      res += buffer;
      res += " ";
      sprintf(buffer, "%.6g", mc[i].b);
      res += buffer;
      res += ", ";
    }
    sprintf(buffer, "%.6g", mc.back().r);
    res += buffer;
    res += " ";
    sprintf(buffer, "%.6g", mc.back().g);
    res += buffer;
    res += " ";
    sprintf(buffer, "%.6g", mc.back().b);
    res += buffer;
  }  

  return res;
}

SFString X3DTK::toSFString(const SFVec2f v)
{
  return SFString::number(v.x) + SFString(" ") 
       + SFString::number(v.y);
}

SFString X3DTK::toSFString(const MFVec2f &mv)
{
  SFString res("");
  char buffer[20];
  
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
    {
      sprintf(buffer, "%.6g", mv[i].x);
      res += buffer;
      res += " ";
      sprintf(buffer, "%.6g", mv[i].y);
      res += buffer;
      res += ", ";
    }
    sprintf(buffer, "%.6g", mv.back().x);
    res += buffer;
    res += " ";
    sprintf(buffer, "%.6g", mv.back().y);
    res += buffer;
  }   
  return res;
}

SFString X3DTK::toSFString(const SFVec3f v)
{
  return SFString::number(v.x) + SFString(" ") 
       + SFString::number(v.y) + SFString(" ")
       + SFString::number(v.z);
}

SFString X3DTK::toSFString(const MFVec3f &mv)
{
  SFString res("");
  char buffer[20];
    
  if (!mv.empty())
  {
    for (unsigned int i = 0; i < mv.size() - 1; ++i)
    {
      sprintf(buffer, "%.6g", mv[i].x);
      res += buffer;
      res += " ";
      sprintf(buffer, "%.6g", mv[i].y);
      res += buffer;
      res += " ";
      sprintf(buffer, "%.6g", mv[i].z);
      res += buffer;
      res += ", ";
    }
    sprintf(buffer, "%.6g", mv.back().x);
    res += buffer;
    res += " ";
    sprintf(buffer, "%.6g", mv.back().y);
    res += buffer;
    res += " ";
    sprintf(buffer, "%.6g", mv.back().z);
    res += buffer;
  }   
  return res;
}

SFString X3DTK::toSFString(const SFRotation r)
{
  return SFString::number(r.x) + SFString(" ") 
       + SFString::number(r.y) + SFString(" ")
       + SFString::number(r.z) + SFString(" ")
       + SFString::number(r.angle);
}

SFString X3DTK::toSFString(const MFRotation &mr)
{
  SFString res("");
  if (!mr.empty())
  {
    for (unsigned int i = 0; i < mr.size() - 1; ++i)
      res += toSFString(mr[i]) + SFString(", ");
    
    res += toSFString(mr.back());        
  }  
  return res;
}
